/************************************************************************************
* Copyright (C) 1997 Charles-Edouard Ruault ce@ruault.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
************************************************************************************/
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;
import java.io.*;
import java.text.*;
import News.*;
import Mail.*;
import utils.Result;

public class ArticleComposer extends JFrame implements ActionListener, Observer
{
	private NewsArticle RefArticle;
	private JMenuBar Bar;
	private JMenuItem post, mail, m_p, show_h;
	private JTextField Subject, To, NewsGroups, Cc, Date;
	private JTextArea Body;
	private JMenu M1, reply;
	private NewsGroupList GroupList;
	private NewsGroup Group;
	private int Type;
	private JSplitPane split;
	private JPanel attachments;
	private HeaderViewer HViewer;
	private ArticleComposer Composer;
	private int _senderCount;
	public final static int NEWS=1, MAIL=2, REPLY=4, VIEW=8;
	
	private String getSignature(String file)
	{
		File src;
		FileInputStream in=null;
		byte tab[];
		int read=-1, length;
		String Desc;
		StringTokenizer tk;

		src=new File(file);
		try
		{
			if (!(src.exists()&&src.canRead()))
				return null;
			in=new FileInputStream(src);
			length=(int)src.length();
			tab=new byte[length];
			read=in.read(tab, 0, length);
			return new String(tab);
		}
		catch (FileNotFoundException fn)
		{
			JNewsDialog err=new JNewsDialog(this, "Error", "File "+file+" not found : "+fn.getMessage());
			return null;
		}
		catch (java.io.IOException ex)
		{
			System.out.println("Exception while reading "+file+" : "+ex.getMessage());
			return null;
		}
		finally
		{
			if (in != null)
				try { in.close(); }
				catch (java.io.IOException e1) {return null;}
		}
	}

	public ArticleComposer(NewsGroupList groupl, NewsGroup group, NewsArticle ref, int type)
	{
		super("Article Composer");

		JLabel lbl;
		GroupList=groupl;
		Group=group;
		Type=type;
		RefArticle=ref;

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e)
            {
                setVisible(false);
            }
        });

		JPanel P=new JPanel();
		GridBagLayout grid=new GridBagLayout();
		GridBagConstraints c=new GridBagConstraints();
		P.setLayout(grid);

		c.weightx=0;
		c.weighty=0;
		c.gridwidth=1;
		c.anchor=GridBagConstraints.EAST;
		c.fill=GridBagConstraints.NONE;
		lbl=new JLabel("Subject :");
		grid.setConstraints(lbl, c);
		P.add(lbl);

		c.gridwidth=GridBagConstraints.REMAINDER;
		c.anchor=GridBagConstraints.WEST;
		c.fill=GridBagConstraints.HORIZONTAL;
		Subject=new JTextField("");
		grid.setConstraints(Subject, c);
		P.add(Subject);
		if (Type==VIEW)
			Subject.setEditable(false);

		c.gridwidth=1;
		c.anchor=GridBagConstraints.EAST;
		c.fill=GridBagConstraints.NONE;
		lbl=new JLabel("Newsgroups :");
		grid.setConstraints(lbl, c);
		P.add(lbl);

		c.gridwidth=GridBagConstraints.REMAINDER;
		c.anchor=GridBagConstraints.WEST;
		c.fill=GridBagConstraints.HORIZONTAL;
		if (((Type&NEWS)!=0)||((Type&VIEW)!=0))
			NewsGroups=new JTextField(group.name());
		else
			NewsGroups=new JTextField("");
		if (Type==VIEW)
			NewsGroups.setEditable(false);
		grid.setConstraints(NewsGroups, c);
		P.add(NewsGroups);

		c.gridwidth=1;
		c.anchor=GridBagConstraints.EAST;
		c.fill=GridBagConstraints.NONE;
		lbl=new JLabel("To :");
		grid.setConstraints(lbl, c);
		P.add(lbl);

		c.gridwidth=GridBagConstraints.REMAINDER;
		c.anchor=GridBagConstraints.WEST;
		c.fill=GridBagConstraints.HORIZONTAL;
		To=new JTextField("");
		grid.setConstraints(To, c);
		P.add(To);
		if (Type==VIEW)
			To.setEditable(false);

		c.gridwidth=1;
		c.anchor=GridBagConstraints.EAST;
		c.fill=GridBagConstraints.NONE;
		lbl=new JLabel("Cc :");
		grid.setConstraints(lbl, c);
		P.add(lbl);

		c.gridwidth=GridBagConstraints.REMAINDER;
		c.anchor=GridBagConstraints.WEST;
		c.fill=GridBagConstraints.HORIZONTAL;
		Cc=new JTextField("");
		grid.setConstraints(Cc, c);
		P.add(Cc);
		if (Type==VIEW)
		{
			Cc.setEditable(false);
			Cc.setText(ref.getHeaderField("Cc"));
		}

		if (Type==VIEW)
		{
			c.gridwidth=1;
			c.anchor=GridBagConstraints.EAST;
			c.fill=GridBagConstraints.NONE;
			lbl=new JLabel("Date :");
			grid.setConstraints(lbl, c);
			P.add(lbl);

			c.gridwidth=GridBagConstraints.REMAINDER;
			c.anchor=GridBagConstraints.WEST;
			c.fill=GridBagConstraints.HORIZONTAL;
			Date=new JTextField(formatDate(ref.getDate()));
			Date.setEditable(false);
			grid.setConstraints(Date, c);
			P.add(Date);
		}

		c.weightx=1;
		c.weighty=1;
		c.gridwidth=GridBagConstraints.REMAINDER;
		c.fill=GridBagConstraints.BOTH;
		Body=new JTextArea("", 30, 80);
		grid.setConstraints(Body, c);
		P.add(Body);
		if (Type==VIEW)
			Body.setEditable(false);
		
		if (((Type&REPLY)!=0)||((Type&VIEW)!=0))
		{
			if ((Type&REPLY)!=0)
			{
				int pos=-1, pos1;
				boolean last=false;
				String OrigBody;
				try
				{
					OrigBody=RefArticle.getBody(Group.getServer());
					do
					{
						pos1=OrigBody.indexOf('\n', pos+1);
						if (pos1==-1)
						{
							last=true;
							pos1=OrigBody.length()-1;
						}
					Body.append(">"+OrigBody.substring(pos+1, pos1+1));
					pos=pos1;
					}
					while (!last);
				}
				catch (Exception ex)
				{
					System.out.println("Unable to get body for this article\n"+ ex.getMessage());
				}
			}
			else
			{
				try
				{
					Body.setText(RefArticle.getBody(Group.getServer()));
				}
				catch (Exception ex)
				{
					System.out.println("Unable to get body for this article\n"+ ex.getMessage());
				}
			}
			String subject=RefArticle.getHeaderField("Subject");
			if (subject.toLowerCase().startsWith("re: ")||((Type&VIEW)!=0))
				Subject.setText(subject);
			else
				Subject.setText("Re: "+subject);
			if (((Type&MAIL)!=0)||((Type&VIEW)!=0))
				To.setText(RefArticle.getHeaderField("From"));
		}

		if ((Type&VIEW)==0)
		{	
			String sig=GroupList.getProperties().getProperty("Signature");
			if (sig != null)
			{
				String signature=getSignature(sig);
				if (signature != null)
					Body.append("\n\n"+signature);
			}
		}

		Bar=new JMenuBar();
		M1=new JMenu("Article");
		JMenuItem item;
		if (Type==VIEW)
		{
			M1.add((show_h=new JMenuItem("Show headers")));
			show_h.addActionListener(this);
			M1.add((reply=new JMenu("Reply To")));
			reply.add((item=new JMenuItem("News")));
			item.addActionListener(this);
			reply.add((item=new JMenuItem("Mail")));
			item.addActionListener(this);
			reply.add((item=new JMenuItem("Mail & News")));
			item.addActionListener(this);
			M1.add(reply);
		}
		else
		{	
			M1.add((item=new JMenuItem("Send")));
			item.addActionListener(this);
			M1.addSeparator();
			M1.add((item=new JMenuItem("Discard")));
			item.addActionListener(this);
		}
		Bar.add(M1);
		setJMenuBar(Bar);
		attachments=new JPanel();
		attachments.add("Center", new JLabel("No attachments yet ."));
		split=new JSplitPane(JSplitPane.VERTICAL_SPLIT, true, P, attachments);
		split.setOneTouchExpandable(true);
		getContentPane().add(split);

		if (Type==VIEW)
		{
			HViewer=new HeaderViewer(RefArticle.getHeaderField("Message-ID"));
			HViewer.setArticle(RefArticle);
		}
		pack();
		show();
		split.setDividerLocation(0.9);
	}

	public void clearText()
	{

		Body.setText("");
		super.setTitle(" ");
		Date.setText("");
		To.setText("");
		NewsGroups.setText("");
		if (HViewer != null)
			HViewer.clearText();
		show_h.setEnabled(false);
		reply.setEnabled(false);
	}

	public void setArticle(NewsArticle sel)
	{

		setTitle(sel.getHeaderField("Subject")+" ( "+Group.name()+" )");
		RefArticle=sel;
		try
		{
			Body.setText(sel.getBody(Group.getServer()));
			HViewer.setArticle(RefArticle);
		}
		catch (Exception ex)
		{
			System.out.println("Exception "+ex.getMessage());
		}
		Date.setText(formatDate(RefArticle.getDate()));
		To.setText(RefArticle.getHeaderField("From"));
		Subject.setText(RefArticle.getHeaderField("Subject"));
		NewsGroups.setText(RefArticle.getHeaderField("Newsgroups"));
		show_h.setEnabled(true);
		reply.setEnabled(true);
		split.setDividerLocation(0.9);
	}

	private NewsArticle ComposeNewsArticle()
	{
		StringBuffer Header;
		JNewsProps Props=GroupList.getProperties();
		Date now;
		NewsArticle ToPost;
		String reference;

		Header=new StringBuffer("From: "+Props.get("EMail")+" ("+Props.get("Name")+")\n");
		Header.append("Organization:"+Props.get("Organization")+"\n");
		Header.append("Subject:"+Subject.getText()+"\n");
		Header.append("Newsgroups: "+NewsGroups.getText()+"\n");
		now=new Date();
		Header.append("Date: "+now.toGMTString()+"\n");
		Header.append("Message-ID: "+Group.getServer().getID()+"\n");
		Header.append("X-Newsreader: JavaNews V"+Props.get("Version")+"\n");
		if ((Type&REPLY)!=0)
		{
			Header.append("References: ");
			if ((reference=RefArticle.getHeaderField("References"))!=null)
				Header.append(reference+" "+RefArticle.getHeaderField("Message-ID")+"\n");
			else
				Header.append(RefArticle.getHeaderField("Message-ID")+"\n");
		}
		ToPost=new NewsArticle(Header.toString(), Body.getText());
		return ToPost;
	}

	private MailMessage ComposeMailMessage()
	{
		StringBuffer Header;
		JNewsProps Props=GroupList.getProperties();
		Date now;
		MailMessage ToMail;

		Header=new StringBuffer("From: "+Props.get("EMail")+" ("+Props.get("Name")+")\n");
		Header.append("To: "+To.getText()+"\n");
		Header.append("Organization:"+Props.get("Organization")+"\n");
		Header.append("Subject:"+Subject.getText()+"\n");
		now=new Date();
		TimeZone tz=TimeZone.getTimeZone("GMT");
		DateFormat df=DateFormat.getDateTimeInstance(DateFormat.LONG, DateFormat.FULL);
		
		Header.append("Date: "+now.toGMTString()+"\n");
		Header.append("X-Mailer: JavaNews V"+Props.get("Version")+"\n");
		ToMail=new MailMessage((String)Props.get("SmtpHost"), (String)Props.get("SmtpPort"), Header.toString(), Body.getText());
		return ToMail;
	}

	public void actionPerformed(ActionEvent e)
	{

		if (e.getSource() instanceof JMenuItem)
		{
		    String cmd=e.getActionCommand();
			if (cmd.equals("Close"))
			{
				HViewer.setVisible(false);
				setVisible(false);
			}
			else if (cmd.equals("News"))
			{
				Composer=new ArticleComposer(GroupList, Group, RefArticle, ArticleComposer.NEWS|ArticleComposer.REPLY);
			}
			else if (cmd.equals("Mail"))
			{
				Composer=new ArticleComposer(GroupList, Group, RefArticle, ArticleComposer.MAIL|ArticleComposer.REPLY);
			}
			else if (cmd.equals("Mail & News"))
			{
				Composer=new ArticleComposer(GroupList, Group, RefArticle, ArticleComposer.NEWS|ArticleComposer.MAIL|ArticleComposer.REPLY);
			}
			else if (/*(MenuItem)e.getSource()==show_h*/cmd.equals("Show Header"))
			{
				HViewer.show();
			}
			else if (cmd.equals("Discard"))
			{
				setVisible(false);
			}
			else if (cmd.equals("Send"))
			{
				NewsArticle ToSend=null;
				MailMessage ToMail=null;
				_senderCount=0;
				if (!NewsGroups.getText().equals(""))
				{
					ToSend=ComposeNewsArticle();
					ToSend.addObserver(this);
					ToSend.Post(Group.getServer());
					_senderCount++;
				}
				if (!To.getText().equals(""))
				{
					ToMail=ComposeMailMessage();
					ToMail.addObserver(this);
					ToMail.Send();
					_senderCount++;
				}
				if (_senderCount>0)
					setEnabled(false);
			}
		}
    }

	public void update(Observable o, Object arg)
	{
		System.out.println("update called ");
		Result res=(Result)arg;
		if (o instanceof MailMessage)
		{
			System.out.println("Mail message");
			if (res.getCode())
			{
				if (--_senderCount==0)
				{
					setEnabled(true);
					setVisible(false);
					dispose();
				}
				new dialog(this, "Mail sent", "Your mail has been sent", null);
			}
			else
			{
				setEnabled(true);
				new dialog(this, "Error sending mail", "An SMTP Error occured", res.getMessage());
			}
		}
		else if (o instanceof NewsArticle)
		{
			if (res.getCode())
			{
				if (--_senderCount==0)
				{
					setEnabled(true);
					setVisible(false);
					dispose();
				}
				new dialog(this, "Message posted", "Your message has been posted", null);
			}
			else
			{
				setEnabled(true);
				new dialog(this, "Error posting message", "An NNTP Error occured", res.getMessage());			
			}
		}
	}

	protected String formatDate(long date)
	{
		return (new SimpleDateFormat ()).format(new Date(date));
	}

	class dialog implements Runnable
	{
		String _title;
		String _msg;
		String _msg1;
		JFrame _parent;
		
		public dialog(JFrame parent, String title, String msg, String msg1)
		{
			_parent=parent;
			_title=title;
			_msg=msg;
			_msg1=msg1;
			SwingUtilities.invokeLater(this);
		}
	
		public void run() 
		{
			if (_msg1!=null)
				new JNewsDialog(_parent, _title, _msg, _msg1);
			else
				new JNewsDialog(_parent, _title, _msg);
		}
	}
}









