/************************************************************************************
* Copyright (C) 1997 Charles-Edouard Ruault ce@ruault.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
************************************************************************************/
package Mail;
import java.net.*;
import java.lang.*;
import java.io.*;
import java.util.*;
import Mail.SmtpException;
import Mail.SmtpResponse;

public class Mailer
{
	private int Port=25;
	private String MailHost;
	private InetAddress HostAddr;
	private Socket Sock;
	private BufferedReader In;
	private PrintWriter Out;
	private MailMessage Message;

	public void connect()
	{

		try {
			HostAddr=InetAddress.getByName(MailHost);
		}
		catch (Exception e) {
			System.err.println("Exception : "+e);
		}
		try {
			Sock=new Socket(HostAddr, Port);
			In=new BufferedReader(new InputStreamReader(Sock.getInputStream()));
			Out=new PrintWriter(Sock.getOutputStream());
			System.out.println("Connected to "+MailHost+":"+Port);
			getResponse();
		}
		catch (Exception ex) {
			System.err.println("Exception : "+ex);
		}
	}


	public void disconnect()
	{

		System.out.println("Disconnected from "+MailHost+":"+Port);
		try
		{
			if (In != null)
			{
				In.close();
				In=null;
			}

			if (Out != null)
			{
				Out.close();
				Out=null;
			}
			if (Sock != null)
			{
				Sock.close();
				Sock=null;
			}
		}
		catch(Exception ex)
		{
			System.out.println("Exception "+ex.getMessage());
		}
	}


	public Mailer(MailMessage message, String Host, int port)
	{

		Message=message;
		MailHost=Host;
		Port=port;
	}

	private SmtpResponse getResponse() throws SmtpException, IOException
	{
		int status;
		SmtpResponse resp;

		//System.out.println("Waiting for response...");
		resp=new SmtpResponse(In.readLine());
		status=resp.getCode();
		if ((status >= 100 && status <= 299)||(status == 354))
			return resp;
		throw new SmtpException(resp.getMessage());
	}

	private void sendCommand(String cmd)
	{
		Out.print(cmd);
		Out.print("\r\n");
		Out.flush();
		//System.out.println("sent cmd : "+cmd);
	}
	

	private String ProcessBody(String Body)
	{
		StringBuffer buff=new StringBuffer("");
		String tmp;
		StringTokenizer token;

		token=new StringTokenizer(Body, "\n", true);
		while (token.hasMoreElements())
		{
			tmp=token.nextToken();
			//System.out.println("Token : "+tmp+" Fin Token");
			if (tmp.startsWith("."))
				buff.append("."+tmp);
			else
				buff.append(tmp);
		}
		return buff.toString();
	}

	private boolean Hello() throws SmtpException, IOException
	{

		sendCommand("helo "+MailHost);
		if (getResponse().getCode() != 250)
			return false;
		if (In.ready())
			getResponse();
		return true;
	}

	private boolean From() throws SmtpException, IOException
	{

		sendCommand("mail from: "+Message.getSender().getMail());
		if (getResponse().getCode() != 250)
			return false;
		return true;
	}

	private boolean To() throws SmtpException, IOException
	{

		sendCommand("rcpt to: "+Message.getReceiver().getMail());
		if (getResponse().getCode() != 250)
			return false;
		return true;
	}

	public boolean Data() throws SmtpException, IOException
	{
		int code;
		String NewBody;

		NewBody=ProcessBody(Message.getBody());
		try {
			sendCommand("data");
			code=getResponse().getCode();
			if (code==354)
			{
				Out.print(Message.getHeader());
				Out.print("\r\n");
				Out.print(NewBody);
				Out.print("\r\n.\r\n");
				Out.flush();
				code=getResponse().getCode();
				return (code==250);
			}
		}
		catch (SmtpException e)
		{
			System.out.println("Exception Data = "+e.getMessage());
			System.out.println("Header : \n"+Message.getHeader());
			System.out.println("\nBody : \n"+NewBody);
			throw e;
		}
		return false;
	}

	private boolean Quit() throws SmtpException, IOException
	{

		sendCommand("quit");
		if (getResponse().getCode() != 221)
			return false;
		return true;
	}

	public boolean Send() throws SmtpException, IOException
	{

		try
		{
			connect();
			if (Hello()==false)
			{
				disconnect();
				return false;
			}
			if (From()==false)
			{
				disconnect();
				return false;
			}
			if (To()==false)
			{
				disconnect();
				return false;
			}
			if (Data()==false)
			{
				disconnect();
				return false;
			}
			Quit();
			disconnect();
		}
		catch (SmtpException e)
		{
			disconnect();
			throw e;
		}
		catch (IOException eio)
		{
			disconnect();
			throw eio;
		}
		return true;
	}

	protected void finalize() throws Throwable
	{

		disconnect();
	}
}


