/************************************************************************************
* Copyright (C) 1998 Charles-Edouard Ruault ce@ruault.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
************************************************************************************/
package Message;
import java.lang.*;
import java.util.*;

public class Header
{
  public static String Mime_Version="mime-version";
  public static String Content_Type="content-type";
  public static String Content_Transfer_Encoding="content-transfer-encoding";
  public static String Base64_Encoded="base64";
  public static String Multipart="multipart";
  public static String Mixed="mixed";
  public static String Text="text";
  public static String Plain="plain";
  public static String Type="__type__";
  public static String Subtype="__subtype__";
  public static String Boundary="boundary";
  public static String Name="name";
  protected static boolean debug=false;
  private Hashtable _fields;

  public Header(String header)
  {
    if (debug)
      System.out.println("new header \n"+header+"\n");
    _fields=parse(header);
  }

  public Header()
  {
    _fields=new Hashtable();
  }

  protected static boolean parseLine(String in, Hashtable fields)
  {
    int idx;
    String key, value;
	  
    idx=in.indexOf(':');
    if (idx == -1)
    {
      System.out.println("Invalid Header : "+in);
      return false;
    }
    key=in.substring(0, idx).toLowerCase();
    if (in.length()>=(idx+2))
      value=in.substring(idx+2);
    else
      value="";
    fields.put(key, value);
    if (debug)
      System.out.println(key+" = "+value);
    return true;
  }

  protected static Hashtable parse(String header)
  {
    int end, tmp;
    String cur;
    char c;
    Hashtable table=new Hashtable();
    
    cur=header;
    end=cur.indexOf("\n");
    while(end!=-1)
    {
      //System.out.println("Current = \n"+cur);
      if (end < (cur.length()-1))
      {
	c=cur.charAt(end+1);
	while ((c==' ')||(c=='\t'))
	{
	  tmp=cur.indexOf("\n", end+1);
	  if (tmp != -1)
	  {
	    end=tmp;
	    if (end < (cur.length()-1))
	      c=cur.charAt(end+1);
	    else
	    {
	      c=1;
	      end=cur.length()-1;
	    }
	  }
	  else
	  {
	    end=cur.length();
	    c=0;
	  }
	}
	if (parseLine(cur.substring(0, end), table)==false)
	  System.out.println("Bad Header :"+header);
	cur=cur.substring(end+1);
	end=cur.indexOf("\n");
      }
      else
      {
	if (parseLine(cur.substring(0, end), table )==false)
	  System.out.println("Bad Header :"+header);
	end=-1;
      }
    }
    return table;
  }

  protected static Hashtable parseValue(String header)
  {
    Hashtable res=new Hashtable();
    String param, value;
    int idx=header.indexOf('/');
    int idx_n;
    if (idx!=-1)
    {
      res.put(Type, header.substring(0, idx++).trim());
      idx_n=header.indexOf(';', idx);
      if (idx_n==-1)
	res.put(Subtype, header.substring(idx).trim().toLowerCase());
      else
      {
	res.put(Subtype, header.substring(idx, idx_n).trim().toLowerCase());
	//now parse additional params
	idx=idx_n+1;
      }
    }
    else
      idx=0;
    while(idx!=-1)
    {
      idx_n=header.indexOf('=', idx);
      if (idx_n==-1)
	break;//error
      param=header.substring(idx, idx_n);
      idx=idx_n+1;
      idx_n=header.indexOf(';', idx);
      if (idx_n==-1)
      {
	value=header.substring(idx);
	idx=idx_n;
      }
      else
      {
	value=header.substring(idx, idx_n);
	idx=idx_n+1;
      }
      res.put(param.trim().toLowerCase(), value.trim());
    }
    return res;
  }

  public void updateMimeHeaders(String boundary)
  {
    if (getField(Mime_Version)==null)
      setField(Mime_Version, "1.0");
    if (getField(Content_Type)==null)
    {
      StringBuffer ctype=new StringBuffer(Multipart);
      ctype.append('/');
      ctype.append(Mixed);
      ctype.append("; ");
      ctype.append(Boundary);
      ctype.append("=\"");
      ctype.append(boundary);
      ctype.append("\"");
      setField(Content_Type, ctype.toString());
    }
  }

  public static String unQuote(String key)
  {
  
    if (key!=null)
    {
      if (key.startsWith("\""))
      {
	key=key.substring(1);
	if (key.endsWith("\""))
	  key=key.substring(0, key.length()-1);
      }
    }
    return key;
  }

  public String getField(String key)
  {
    
    return (String)_fields.get(key);
  }

  public void setField(String key, String value)
  {
    _fields.put(key, value);
  }

  public Enumeration getFields()
  {
    return _fields.elements();
  }
  
  public String toString()
  {
    StringBuffer sb=new StringBuffer();
    Enumeration en=_fields.keys();
    String key;
    while(en.hasMoreElements())
    {
      key=(String)en.nextElement();
      sb.append(key);
      sb.append(": ");
      sb.append((String)_fields.get(key));
      sb.append('\n');
    }
    return sb.toString();
  }
}







