/************************************************************************************
* Copyright (C) 1998 Charles-Edouard Ruault ce@ruault.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
************************************************************************************/
package Message;
import java.lang.*;
import java.util.*;
import java.io.*;

public class Message
{
	public static boolean debug=false;
	private Body _body;
	private Header _header;
	private String _boundary;
	private boolean _isMime=false;


	public Message(String message) throws MessageException
	{
		int idx=message.indexOf("\n\n");
		if (idx==-1)
			throw new MessageException("malformed message ( missing header )");
		_header=new Header(message.substring(0, idx+1));
		processMime(message.substring(idx+2));
	}

	public Message(String header, String body) throws MimeException
	{
		_header=new Header(header);
		processMime(body);
	}

	public Message()
	{
		_header=new Header();
		_body=new Body();
		_isMime=false;
	}

	public void setHeaders(Header header)
	{
		_header=header;
	}

	public String getHeaderField(String key)
	{
		if (_header==null)
			return null;
		return _header.getField(key);
	}

	public void setHeaderField(String key, String value)
	{
		if (_header==null)
			_header=new Header();
		_header.setField(key, value);
	}

	public String getHeader()
	{
		if (_header==null)
			return "";
		return _header.toString();
	}

	public String getBody() throws MessageException
	{
		if (_body==null)
			return "";
		if (hasParts())
		{
			_isMime=true;
			if (_boundary==null)
			{
				_boundary=generateBoundary();
				_header.updateMimeHeaders(_boundary);
			}
			return _body.buildUp(_boundary).toString();
		}
		else
			return _body.buildUp().toString();
	}

	public void setBody(String body) throws MessageException
	{
		if (_header==null)
			throw new MessageException("Unable to process Body, no headers defined");
		processMime(body);
	}

	public void addPart(char data[], Header headers) throws MessageException
	{
		if (_header==null)
			throw new MessageException("Unable to add a part, no headers defined");
		_body.addPart(new MimePart(data, headers));
	}

	public void addPart(MimePart part)
	{
		_body.addPart(part);
	}

	public void attachFile(String name)
	{
		 File in_f=new File(name);
		 int len=(int)in_f.length();
		 char data[]=new char[len];
		 BufferedReader in=new BufferedReader(new FileReader(in_f));
		 in.read(data, 0, len);
		 in.close();
		 //Need to get the content type ...
		 m.addPart(new MimePart("image", "gif", in_f.getName(), data));
	}

	public boolean hasParts()
	{
		if (_body==null)
			return false;
		return _body.hasParts();
	}

	public Enumeration getParts()
	{
		if (_body==null)
			return null;
		return _body.getParts();
	}

	public Enumeration getHeaderFields()
	{
		if (_header==null)
			return null;
		return _header.getFields();
	}

	public StringBuffer getMessage() throws MessageException
	{
		if (_header==null)
			throw new MessageException("No headers defined for this message");
		if (hasParts())
		{
			_boundary=generateBoundary();
			_header.setField(Header.Mime_Version, "1.0");
			_header.setField(Header.Content_Type, Header.Multipart+"/"+Header.Mixed+"; "+Header.Boundary+"=\""+_boundary+"\"");
		}
		StringBuffer sb=new StringBuffer(_header.toString());
		sb.append("\n\n");
		sb.append(getBody());
		return sb;
	}

	protected String generateBoundary()
	{
		StringBuffer sb=new StringBuffer("__CER");
		sb.append(Long.toHexString(System.currentTimeMillis()));
		sb.append("CER__");
		return sb.toString();
	}

	protected void processMime(String body) throws MimeException
	{
		if (getHeaderField(Header.Mime_Version)==null)
		{
			_body=new Body(body);
			_isMime=false;
			if (debug)
				System.out.println("Not a MIME message");
		}
		else
		{
			_isMime=true;
			if (debug)
				System.out.println("Mime message...");
			String type=getHeaderField(Header.Content_Type);
			if (type==null)
				throw new MimeException("Missing Content-type header");
			Hashtable value=Header.parseValue(type);
			if (debug)
			{
				Enumeration en=value.keys();
				while(en.hasMoreElements())
				{
					String key=(String)en.nextElement();
					System.out.println(key+" = "+value.get(key));
				}
			}
			String mime_type=(String)value.get(Header.Type);
			_boundary=Header.unQuote((String)value.get(Header.Boundary));
			_body=new Body(_header, body);
		}
	}
}







