/************************************************************************************
* Copyright (C) 1997 Charles-Edouard Ruault ce@ruault.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
************************************************************************************/
package News;
import java.util.*;

class IntBag {
	private Vector vector;

	public IntBag() {
		vector = new Vector();
	}

	public IntBag(String line) throws Exception {
		vector = new Vector();
		int i;

		if (line.equals("")) {
			return;
		}
		StringTokenizer st = new StringTokenizer(line.substring(1, line.length()),",",false);
		//System.err.println("String-based IntBag Constructor");

		while (st.hasMoreTokens()) {
			String s = st.nextToken();
			if ((i=s.indexOf("-")) == -1) {
				Integer value = new Integer(s);
				insert(value.intValue());
			}
			else {
				Integer val1=new Integer(s.substring(0,i));
				Integer val2=new Integer(s.substring(i+1,s.length()));
				insert(new Interval(	val1.intValue(),
										val2.intValue()));
			}
		}
	}

	public void insert(int elem) throws Exception {
		insert(new Interval(elem));
	}

	public void insert(Interval target) throws Exception {
		Enumeration elements = vector.elements();

		while (elements.hasMoreElements()) {
			int current = vector.indexOf(elements.nextElement());
			if (target.lesserThan((Interval) vector.elementAt(current))) {
				vector.insertElementAt(target,current);
				rearrangeOnInsert(current);
				return;
			}
		}
		vector.addElement(target);
		rearrangeOnInsert(vector.size()-1);
	}

	public void remove(int elem) throws Exception {
		remove(new Interval(elem));
	}

	public void remove(Interval target) throws Exception {
		Enumeration elements = vector.elements();

		while (elements.hasMoreElements()) {
			Interval currentInterval = (Interval) elements.nextElement();
			int current = vector.indexOf(currentInterval);
			if (target.within(currentInterval)) {
				vector.removeElementAt(current);
				if (target.inf() > currentInterval.inf()) {
					vector.insertElementAt(	new Interval(currentInterval.inf(),
														 target.inf()-1),
											current++);
				}
				if (currentInterval.sup() > target.sup()) {
					vector.insertElementAt(	new Interval(target.sup()+1,
														 currentInterval.sup()),
											current);
				}
				return;
			}

		}
	}

	private void rearrangeOnInsert(int index) {
		int myi=index;

		try {
			Interval current = (Interval) vector.elementAt(myi);
			if (myi > 0) {
				Interval previous = (Interval) vector.elementAt(myi-1);
				if (current.contiguouslyGreaterThan(previous)) {
					Interval newInterval = new Interval(previous, current);
					vector.removeElementAt(myi);
					vector.removeElementAt(myi-1);
					vector.insertElementAt(newInterval, --myi);
					current = (Interval) vector.elementAt(myi);
				}
				else if (current.within(previous)) {
					vector.removeElementAt(myi);
					//System.err.println("Rearrange: remove within");
					return;
				}
			}
			if (myi < vector.size()-1) {
				Interval next = (Interval) vector.elementAt(myi+1);
				if (current.contiguouslyLesserThan(next)) {
					Interval newInterval = new Interval(current, next);
					vector.removeElementAt(myi);
					vector.removeElementAt(myi);
					vector.insertElementAt(newInterval, myi);
				}
			}
		}
		catch (Exception e) {
			System.err.println("Rearrange Exception: "+e);
		}
		return;
	}

	public Vector vector() {
		return vector;
	}

	public String toString() {
		StringBuffer sb = new StringBuffer();
		Enumeration elements = vector.elements();

		if (elements.hasMoreElements()) {
			sb.append(" "+elements.nextElement().toString());
		}
		while (elements.hasMoreElements()) {
			sb.append(",").append(elements.nextElement().toString());
		}
		return sb.toString();
	}

	public boolean includes(int elem) {
		Enumeration elements = vector.elements();
		Interval interval = new Interval(elem);

		while (elements.hasMoreElements()) {
			if (interval.within((Interval) elements.nextElement())) {
				return true;
			}
		}
		return false;
	}
}
