/************************************************************************************
* Copyright (C) 1997 Charles-Edouard Ruault ce@ruault.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
************************************************************************************/
package News;
import java.lang.*;
import java.util.*;
import java.io.IOException;
import utils.*;

public class NewsArticle extends Observable implements utils.Ordered, Runnable
{
	private String Body;
	private String Header;
	private Hashtable HeaderFields;
	private boolean BodyPresent, Read;
	private int Index;
	private long date=0;
	private String ID=null;
	private Thread sender;
	private NewsServer _server;
	
	
	private boolean ParseFields(String in)
	{
		int idx;
		String key, value;

		idx=in.indexOf(':');
		if (idx == -1)
		{
			System.out.println("Invalid Header : "+in);
			return false;
		}
		key=in.substring(0, idx);
		if (in.length()>=(idx+2))
			value=in.substring(idx+2);
		else
			value="";
		HeaderFields.put(key, value);
		//System.out.println("Champ : "+key+" = "+value);
		return true;
	}

	private void ParseHeader()
	{
		int end, tmp;
		String cur;
		char c;

		cur=Header;
		end=cur.indexOf("\n");
		while(end!=-1)
		{
			//System.out.println("Current = \n"+cur);
			if (end < (cur.length()-1))
			{
	    		c=cur.charAt(end+1);
				while ((c==' ')||(c=='\t'))
				{
					tmp=cur.indexOf("\n", end+1);
					if (tmp != -1)
					{
						end=tmp;
						if (end < (cur.length()-1))
							c=cur.charAt(end+1);
						else
						{
							c=1;
							end=cur.length()-1;
						}
					}
					else
					{
						end=cur.length();
						c=0;
					}
				}
				if (ParseFields(cur.substring(0, end))==false)
					System.out.println("Bad Header :"+Header);
				cur=cur.substring(end+1);
				end=cur.indexOf("\n");
			}
			else
			{
				if (ParseFields(cur.substring(0, end))==false)
					System.out.println("Bad Header :"+Header);
				end=-1;
			}
		}

		String Df=getHeaderField("Date");
		if (Df==null)
		{
			System.out.println("No Date field in :"+Header);
			date=(new /*MDate*/Date()).getTime();
		}
		else
		{
			try
			{
				date=/*MDate*/Date.parse(Df);
			}
			catch (Exception e)
			{
				System.out.println("Exception Date() with date : "+Df+"\nMessage : "+e.getMessage());
				date=(new /*MDate*/Date()).getTime();
			}
		}
		ID=getHeaderField("Message-ID");
	}

	public NewsArticle(String header, String body, int index, boolean read)
	{

	    Index=index;
	    Header=new String(header);
		Body=new String(body);
		BodyPresent=true;
		Read=read;
		HeaderFields=new Hashtable();
		ParseHeader();
	}

	public NewsArticle(String header, int index, boolean read)
	{

		Index=index;
		Header=new String(header);
		BodyPresent=false;
		Read=read;
		HeaderFields=new Hashtable();
		ParseHeader();
	}

	public NewsArticle(String header, String body)
	{

		Header=new String(header);
		Body=new String(body);
		BodyPresent=true;
		Read=false;
		HeaderFields=new Hashtable();
		ParseHeader();
	}

	public String getHeaderField(String key)
	{

		return (String)HeaderFields.get(key);
	}

	public String getHeader()
	{

	    return Header;
	}

	public String getBody(NewsServer Server) throws Exception
	{

		if (BodyPresent==false)
		{
			Body=new String(Server.getBody(Index));
			BodyPresent=true;
		}
		return Body;
	}

	public int getIndex()
	{

		return Index;
	}

	public void markRead()
	{

		Read=true;
	}

	public Enumeration getHeaderFields()
	{

		return HeaderFields.keys();
	}

	public long getOrderValue()
	{

		return date;
	}

	public boolean isGreater(Object value)
	{

		return (date>((NewsArticle)value).getDate());
	}

	public boolean isSmaller(Object value)
	{

		return (date<((NewsArticle)value).getDate());
	}

	public boolean isEqual(Object value)
	{

		return (date==((NewsArticle)value).getDate());
	}

	public long getDate()
	{

		return date;
	}

	public String getID()
	{

		return ID;
	}

	public void Post(NewsServer Server)
	{
		_server=Server;
		sender=new Thread(this, "Article poster");
		sender.start();
	}

	public void cancel()
	{
	}
	
	public void run()
	{
		boolean sent=false;
		String msg="";
		
		try
		{
			_server.Post(Header, Body);
			sent=true;
		}
		catch (IOException io)
		{
			io.printStackTrace();
			msg=io.getMessage();
		}
		catch (NntpException nntp)
		{
			nntp.printStackTrace();
			msg=nntp.getMessage();
		}
		setChanged();
		notifyObservers(new Result(sent, msg));
	}

	/**
	 * Warning the DefaultListModel used in GroupViewer
	 * assumes that a NewsArticle returns the Subject header
	 * when the toString() method is called.
	 */
	public String toString()
	{
		String res= getHeaderField("Subject");
		if (res!=null)
			return res;
		else 
			return "No subject !";
	}
}






