/************************************************************************************
* Copyright (C) 1997 Charles-Edouard Ruault ce@ruault.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
************************************************************************************/
package News;
import java.lang.*;
import java.util.*;
import News.IntBag;
import utils.*;

public class NewsGroup implements Ordered
	{
	private String Host;
	private int Port;
	private String GroupName;
	private boolean Subscribed, Posting;
	private int First=0, Last=0, maxArticles=100, Start, LastRead=0;
	private Hashtable articlesRef=null;
	private OrderedFolder articles=null;
	private IntBag readList;
	private boolean hasRead=false, Active=false;
	private NewsServer Serveur=null;

	public NewsGroup(String host, int port, String name, int last, int first, String posting)
		{

		GroupName=new String(name);
		First=first;
		Last=last;
		LastRead=0;
		Host=host;
		Port=port;
		if (posting.equals("y"))
			Posting=true;
		else
			Posting=false;
		}

	public NewsGroup(String host, int port, String resource)
		{
		int idx;

		Host=host;
		Port=port;
		idx=resource.indexOf(':');
		if (idx == -1)
			{
			idx=resource.indexOf('!');
			if (idx !=-1)
				Subscribed=false;
			}
		else
			{
			Subscribed=true;
			}

		if (idx==-1)
			{
			Subscribed=false;
			GroupName=new String(resource);
			readList=new IntBag();
			}
		else
			{
			GroupName=new String(resource.substring(0, idx));
			hasRead=true;
			try
				{
				readList=new IntBag(resource.substring(idx+1));
				}
			catch (Exception e)
				{
				System.out.println("IntBag Exception :"+e);
				readList=new IntBag();
				}
			}
		LastRead=0;
		}

	public void setIndexes(int first, int last)
		{

		First=first;
		Last=last;
		}

	public void updateIndexes(String val) throws InterruptedException
		{
		StringTokenizer st;

		st=new StringTokenizer(val, " ");
		st.nextToken();
		First=(new Integer(st.nextToken())).intValue();
		Last=(new Integer(st.nextToken())).intValue();
		if ((First+maxArticles)>Last)
			Start=First;
		else
			Start=Last-maxArticles;
		System.out.println("Update : "+GroupName+" first="+First+" last="+Last+" Debut a "+Start);
		if (Last<Start)
			return;
		retrieveArticles(Start, Last);
		}

	public void updateIndexes(int first, int last)  throws InterruptedException
		{

		First=first;
		Last=last;
		if ((First+maxArticles)>Last)
			Start=First;
		else
			Start=Last-maxArticles;
		System.out.println("Update : "+GroupName+" first="+First+" last="+Last+" Debut a "+Start);
		if (Last<Start)
			return;
		retrieveArticles(Start, Last);
		}

	public void Update() throws java.io.IOException, NntpException, InterruptedException
		{
		String ret;

		System.out.println("Update : "+GroupName);
		ret=Serveur.selectGroup(GroupName);
		updateIndexes(ret);
		}

	public synchronized boolean Active()
		{

		if (Active==true)
			return false;
		try
			{
			Serveur=new NewsServer(Host, Port);
			Serveur.connect();
			}
		catch (Exception e)
			{
			Serveur.disconnect();
			return false;
			}
		Active=true;
		return true;
		}

	public synchronized void Inactive()
		{

		if (Active==true)
			{
			Active=false;
			try
				{
				Serveur.Quit();
				Serveur.disconnect();
				}
			catch (Exception e)
				{
				}
			finally
				{
				Serveur=null;
				}
			}
		}

	public NewsServer getServer()
		{

		return Serveur;
		}

	public String name()
		{

		return GroupName;
		}

	public void subscribe()
		{

		Subscribed=true;
		}

	public void unsubscribe()
		{

		Subscribed=false;
		}

	public boolean subscribed()
		{

		return Subscribed;
		}

	public void retrieveArticles(int first, int last) throws InterruptedException
		{
		NewsArticle cur;
		int i;

		if (articles == null)
			{
			articles=new OrderedFolder();
			articlesRef=new Hashtable();
			}
		if (first>=last)
			return;
		if (LastRead>last)
			{
			LastRead=last;
			return;
			}
		if (first>LastRead)
			LastRead=first;

		//System.out.println("Retrieving articles, starting at : "+LastRead+" ( "+first+", "+last+" )\n");
		Date dt=new Date();
		long debut=dt.getTime();
		//System.out.println("Depart a : "+dt.toString());
		while(LastRead<=Last)
		{
			try
				{
				cur=new NewsArticle(Serveur.getHeader(LastRead), LastRead, false);
				articles.insert(cur);
				articlesRef.put(cur.getID(), cur);
				}
			catch(InterruptedException intex)
			{
			    System.out.println("Operation aborted at article #"+LastRead);
			    throw(intex);
			}
			catch (Exception ne)
			{
				System.out.println("retrieveArticles : article "+LastRead+" : "+ne);
			}
			LastRead++;
			java.lang.Thread.yield();
		}
		dt=new Date();
		long fin=dt.getTime();
		System.out.println("Duree : "+String.valueOf((fin-debut)/1000)+" sec");
		//System.out.println("Lastread="+LastRead);
		}

	public Enumeration listArticles()
		{

		return articles.elements();
		}

	public void setArticleRead(NewsArticle article)
		{

		article.markRead();
		try
			{
			readList.insert(article.getIndex());
			}
		catch (Exception e)
			{
			System.out.println("IntBag Insert Exception :"+e);
			}
		hasRead=true;
		}


	public NewsArticle getArticleByID(String ID)
		{
		NewsArticle res;

		res=(NewsArticle)articlesRef.get(ID);
		if (res != null)
			{
			res.markRead();
			hasRead=true;
			}
		return res;
		}

	public void setReadList(String list, boolean subscribed)
		{

		if (subscribed)
			subscribe();
		else
			unsubscribe();
		try
			{
			readList=new IntBag(list);
			}
		catch (Exception e)
			{
			System.out.println("IntBag Exception :"+e);
			readList=new IntBag();
			}
		hasRead=true;
		//System.out.println("setreadlist "+GroupName+" "+list);
		}

	public boolean hasRead()
	{

		return hasRead;
	}

	public String toString()
	{

		if (subscribed())
			return new String(GroupName+":"+ (readList==null ? "":readList.toString()));
		else
			return new String(GroupName+"!"+(readList==null ? "":readList.toString()));
	}

	public long getOrderValue()
	{

		return GroupName.hashCode();
	}

	public boolean isGreater(Object value)
	{

		return (GroupName.compareTo(((NewsGroup)value).name())>0);
	}

	public boolean isSmaller(Object value)
	{

		return (GroupName.compareTo(((NewsGroup)value).name())<0);
	}

	public boolean isEqual(Object value)
		{

		return (GroupName.equals(((NewsGroup)value).name()));
		}

	}
